@@ -30,6 +30,7 @@ class CmdResult():
 logging.basicConfig(stream=sys.stderr)
 logging.getLogger("TestRPC").setLevel(logging.DEBUG)
 logging.getLogger("_cmd").setLevel(logging.DEBUG)
+logging.getLogger("pyonep.onep").setLevel(logging.ERROR)
 log = logging.getLogger("_cmd")
 
 
@@ -37,7 +38,7 @@ def _cmd(argv, stdin):
     '''Runs an exoline command, translating stdin from
     string and stdout to string. Returns a CmdResult.'''
     if True:
-        log.debug(' '.join(argv))
+        log.debug(' '.join([str(a) for a in argv]))
         if stdin is not None:
             log.debug('    stdin: ' + stdin)
     if type(stdin) is str:
@@ -57,7 +58,7 @@ def _cmd(argv, stdin):
     stderr.seek(0)
     stderr = stderr.read().strip()
     if exitcode != 0:
-        log.debug("Exit code was {}".format(exitcode))
+        log.debug("Exit code was {0}".format(exitcode))
     return CmdResult(exitcode, stdout, stderr)
 
 
@@ -93,8 +94,8 @@ class TestRPC(TestCase):
     RE_RID = '[0-9a-f]{40}'
     def _rid(self, s):
         '''Parse rid from s, raising an exception if it doesn't validate.'''
-        m = re.match("^({}).*".format(self.RE_RID), s)
-        self.assertFalse(m is None, "rid: {}".format(s))
+        m = re.match("^({0}).*".format(self.RE_RID), s)
+        self.assertFalse(m is None, "rid: {0}".format(s))
         return str(m.groups()[0])
 
     def _createMultiple(self, cik, resList):
@@ -129,7 +130,7 @@ class TestRPC(TestCase):
                 res = resList[i]
                 if res.alias is not None:
                     pyonep.map(cik, resList[i].rid, res.alias, defer=True)
-                self.l("Created {}, rid: {}".format(res.type, res.rid))
+                self.l("Created {0}, rid: {1}".format(res.type, res.rid))
 
         # map to aliases
         if pyonep.has_deferred(cik):
@@ -148,16 +149,17 @@ class TestRPC(TestCase):
                 '-',
                 *alias,
                 stdin=json.dumps(res.desc))
-        self.assertTrue(r.exitcode == 0, 'create succeeds')
+        self.l(r.stdout)
+        self.assertEqual(r.exitcode, 0, 'create succeeds')
 
-        rid = re.match('rid: ({})'.format(self.RE_RID), r.stdout).groups()[0]
+        rid = re.match('rid: ({0})'.format(self.RE_RID), r.stdout).groups()[0]
         ri = rpc('info', res.parentcik, rid)
         info = json.loads(ri.stdout.strip())
         res.created(rid, info)
 
         # test that description contains what we asked for
         self.l('''Comparing keys.
-Asked for desc: {}\ngot desc: {}'''.format(res.desc, res.info['description']))
+Asked for desc: {0}\ngot desc: {1}'''.format(res.desc, res.info['description']))
         for k, v in res.desc.iteritems():
             if k != 'limits':
                 self.l(k)
@@ -166,13 +168,13 @@ Asked for desc: {}\ngot desc: {}'''.format(res.desc, res.info['description']))
                     'created resource matches spec')
 
         if res.type == 'client':
-            m = re.match('^cik: ({})$'.format(self.RE_RID), r.stdout.split('\n')[1])
+            m = re.match('^cik: ({0})$'.format(self.RE_RID), r.stdout.split('\n')[1])
             self.l(r.stdout)
             self.assertTrue(m is not None)
             cik = m.groups()[0]
             self.assertTrue(res.info['key'] == cik)
 
-        self.l("Created {}, rid: {}".format(res.type, res.rid))
+        self.l("Created {0}, rid: {1}".format(res.type, res.rid))
         return res
 
     def l(self, s):
@@ -196,40 +198,31 @@ Asked for desc: {}\ngot desc: {}'''.format(res.desc, res.info['description']))
         self._createMultiple(self.portalcik, [self.client])
 
         # test details for create, read, and write tests.
-        self.resources = [
-            Resource(
-                self.client.cik(),
-                'dataport',
-                {'format': 'integer', 'name': 'int_port'},
-                write=['-1', '0', '100000000'],
-                record=[[665366400, '42']]),
-            Resource(
-                self.client.cik(),
-                'dataport',
-                {'format': 'boolean', 'name': 'boolean_port'},
-                write=['false', 'true', 'false'],
-                record=[[-100, 'true'], [-200, 'false'], [-300, 'true']]),
-            Resource(
-                self.client.cik(),
-                'dataport',
-                {'format': 'string', 'name': 'string_port'},
-                write=['test', 'a' * 300],
-                record=[[163299600, 'home brew'], [543212345, 'nonsense']],
-                alias='string_port_alias'),
-            Resource(
-                self.client.cik(),
-                'dataport',
-                {'format': 'float', 'name': 'float_port'},
-                write=['-0.1234567', '0', '3.5', '100000000.1'],
-                record=[[-100, '-0.1234567'], [-200, '0'], [-300, '3.5'], [-400, '10000000.1']]),
-                # TODO: handle scientific notation from OneP '-0.00001'
-            Resource(
-                self.client.cik(),
-                'dataport',
-                {'format': 'binary', 'name': 'binary_port'})
-        ]
+        cik = self.client.cik()
+        self.dataports = {}
+        self.dataports['integer'] = Resource(
+            cik, 'dataport', {'format': 'integer', 'name': 'int_port'},
+            write=['-1', '0', '100000000'],
+            record=[[665366400, '42']])
+        self.dataports['boolean'] = Resource(
+            cik, 'dataport', {'format': 'boolean', 'name': 'boolean_port'},
+            write=['false', 'true', 'false'],
+            record=[[-100, 'true'], [-200, 'false'], [-300, 'true']])
+        self.dataports['string'] = Resource(
+            cik, 'dataport', {'format': 'string', 'name': 'string_port'},
+            alias='string_port_alias',
+            write=['test', 'a' * 300],
+            record=[[163299600, 'home brew'], [543212345, 'nonsense']])
+        self.dataports['float'] = Resource(
+            cik, 'dataport', {'format': 'float', 'name': 'float_port'},
+            write=['-0.1234567', '0', '3.5', '100000000.1'],
+            record=[[-100, '-0.1234567'], [-200, '0'], [-300, '3.5'], [-400, '10000000.1']])
+            # TODO: handle scientific notation from OneP '-0.00001'
+        # TODO: handle binary dataport
 
-        self._createMultiple(self.client.cik(), self.resources)
+        self.resources = self.dataports.values()
+
+        self._createMultiple(cik, self.resources)
 
     def tearDown(self):
         '''Clean up any test client'''
@@ -239,7 +232,7 @@ Asked for desc: {}\ngot desc: {}'''.format(res.desc, res.info['description']))
         r = rpc('read',
                 res.parentcik,
                 res.rid,
-                '--limit={}'.format(limit),
+                '--limit={0}'.format(limit),
                 '--timeformat=unix')
         lines = r.stdout.split('\n')
         vread = []
@@ -254,8 +247,8 @@ Asked for desc: {}\ngot desc: {}'''.format(res.desc, res.info['description']))
 
     def _verifyWrite(self, wrotevalues, readvalues):
         readvalues_notime = [v[1] for v in readvalues]
-        self.l('Wrote {}'.format(wrotevalues))
-        self.l('Read  {}'.format(readvalues))
+        self.l('Wrote {0}'.format(wrotevalues))
+        self.l('Read  {0}'.format(readvalues))
         self.assertTrue(wrotevalues == readvalues_notime,
                         'Read values did not match written values')
 
@@ -297,9 +290,9 @@ Asked for desc: {}\ngot desc: {}'''.format(res.desc, res.info['description']))
         wv_errors = sorted(wv_errors, key=lambda x: x[0][0])
 
         # compare arrays
-        self.l('Wrote     {}'.format(wrotevalues))
-        self.l('wv_errors {}'.format(wv_errors))
-        self.l('Read      {}'.format(readvalues))
+        self.l('Wrote     {0}'.format(wrotevalues))
+        self.l('wv_errors {0}'.format(wv_errors))
+        self.l('Read      {0}'.format(readvalues))
         if len(readvalues) != len(wrotevalues):
             return False
         for ((wt, terr), wv), (rt, rv) in zip(wv_errors, readvalues):
@@ -330,7 +323,7 @@ Asked for desc: {}\ngot desc: {}'''.format(res.desc, res.info['description']))
                 r = rpc('record',
                         res.parentcik,
                         res.rid,
-                        '--value={},{}'.format(timestamp, value))
+                        '--value={0},{1}'.format(timestamp, value))
                 self.assertTrue(r.exitcode == 0)
                 time.sleep(1)
 
@@ -338,7 +331,7 @@ Asked for desc: {}\ngot desc: {}'''.format(res.desc, res.info['description']))
             r = rpc('record',
                     res.parentcik,
                     res.rid,
-                    *['--value={},{}'.format(t, v) for t, v in res.record])
+                    *['--value={0},{1}'.format(t, v) for t, v in res.record])
             self.assertTrue(r.exitcode == 0)
 
         def on_stdin(res):
@@ -346,7 +339,7 @@ Asked for desc: {}\ngot desc: {}'''.format(res.desc, res.info['description']))
                     res.parentcik,
                     res.rid,
                     '-',
-                    stdin='\n'.join(['{},{}'.format(t, v) for t, v in res.record]))
+                    stdin='\n'.join(['{0},{1}'.format(t, v) for t, v in res.record]))
             self.assertTrue(r.exitcode == 0)
 
         for r in self.resources:
@@ -367,7 +360,7 @@ Asked for desc: {}\ngot desc: {}'''.format(res.desc, res.info['description']))
         # starts with cik
         self.l(r.stdout)
         self.assertTrue(
-            re.match("cik: {}.*".format(cik), r.stdout) is not None)
+            re.match("cik: {0}.*".format(cik), r.stdout) is not None)
         # has correct number of lines
         self.assertTrue(len(r.stdout.split('\n')) == len(self.resources) + 1)
 
@@ -414,95 +407,79 @@ Asked for desc: {}\ngot desc: {}'''.format(res.desc, res.info['description']))
             "public": False})
         self._create(client)
 
-        # test details for create, read, and write tests.
+        # set up a few standard dataports
+        cik = client.cik()
+        dataports = {}
         resources = [
-            Resource(
-                client.cik(),
-                'dataport',
-                {'format': 'integer', 'name': 'int_port'},
-                write=['-1', '0', '100000000'],
-                record=[[665366400, '42']]),
-            Resource(
-                client.cik(),
-                'dataport',
-                {'format': 'boolean', 'name': 'boolean_port'},
-                write=['false', 'true', 'false'],
-                record=[[-100, 'true'], [-200, 'false'], [-300, 'true']]),
-            Resource(
-                client.cik(),
-                'dataport',
-                {'format': 'string', 'name': 'string_port'},
-                write=['test', 'a' * 300],
-                record=[[163299600, 'home brew'], [543212345, 'nonsense']]),
-            Resource(
-                client.cik(),
-                'dataport',
-                {'format': 'float', 'name': 'float_port'},
-                write=['-0.1234567', '0', '3.5', '100000000.1'],
-                record=[[-100, '-0.1234567'], [-200, '0'], [-300, '3.5'], [-400, '10000000.1']]),
-                # TODO: handle scientific notation from OneP '-0.00001'
-            Resource(
-                client.cik(),
-                'dataport',
-                {'format': 'binary', 'name': 'binary_port'})]
-
+            Resource(cik, 'dataport', {'format': 'integer', 'name': 'int_port'}),
+            Resource(cik, 'dataport', {'format': 'boolean', 'name': 'boolean_port'}),
+            Resource(cik, 'dataport', {'format': 'string', 'name': 'string_port'}),
+            Resource(cik, 'dataport', {'format': 'float', 'name': 'float_port'}),
+        ]
         for res in resources:
             self._create(res)
 
         r = rpc('listing', client.cik(), '--type=dataport', '--plain')
-        self.assertTrue(len(r.stdout.split('\n')) == len(resources), 'all dataports listed')
+
+        lines = r.stdout.split()
+        lines.sort()
+        rids = [r.rid for r in resources]
+        rids.sort()
+        self.l("{0} {1}".format(lines, rids))
+        self.assertTrue(lines == rids, 'listing after create')
         r = rpc('drop', client.cik(), '--all-children')
-        self.assertTrue(r.exitcode == 0, 'drop --all-children succeeded')
+        self.assertEqual(r.exitcode, 0, 'drop --all-children succeeded')
         r = rpc('listing', client.cik(), '--type=dataport', '--plain')
         self.assertTrue(len(r.stdout) == 0), 'no dataports after drop --all-children'
         r = rpc('drop', self.portalcik, client.rid)
-        self.assertTrue(r.exitcode == 0, 'drop client succeeded')
+        self.assertEqual(r.exitcode, 0, 'drop client succeeded')
         r = rpc('info', self.portalcik, client.rid)
         self.assertTrue(r.exitcode != 0 and r.stderr.endswith('restricted'), 'client gone after drop')
 
 
     def spark_test(self):
-        '''Spark chart command'''
+        '''Intervals command'''
         cik = self.client.cik()
         rid = self._rid(
             rpc('create', cik, '--type=dataport', '--format=integer', '--ridonly').stdout)
-        rpc('record', cik, rid, '--interval={}'.format(240), *['--value={}'.format(x) for x in range(1, 6)])
-        r = rpc('spark', cik, rid, '--days=1')
+        rpc('record', cik, rid, '--interval={0}'.format(240), *['--value={0}'.format(x) for x in range(1, 6)])
+        r = rpc('intervals', cik, rid, '--days=1')
         m = re.match("[^ ] {59}\n4m", r.stdout)
         self.assertTrue(m is not None, "equally spaced points")
         rpc('flush', cik, rid)
-        r = rpc('spark', cik, rid, '--days=1')
+        r = rpc('intervals', cik, rid, '--days=1')
         self.assertTrue(r.exitcode == 0 and r.stdout == '', "no data should output nothing")
         r = rpc('record', cik, rid, '--value=-1,1', '--value=-62,2', '--value=-3662,3', '--value=-3723,4')
         self.assertTrue(r.exitcode == 0, "record points")
-        r = rpc('spark', cik, rid, '--days=1')
-        self.l(u'stdout: {} ({})'.format(r.stdout, len(r.stdout)))
+        r = rpc('intervals', cik, rid, '--days=1')
+        self.l(u'stdout: {0} ({1})'.format(r.stdout, len(r.stdout)))
         m = re.match("^[^ ] {58}[^ ]\n1m 1s +1h$", r.stdout)
         self.assertTrue(m is not None, "three points, two intervals")
 
     def _latest(self, cik, rid, val, msg):
         r = rpc('read', cik, rid, '--format=raw')
-        self.l(r.stdout)
-        self.assertTrue(r.stdout == val, msg)
+        self.assertEqual(r.exitcode, 0, 'read succeeded')
+        self.l("{0} vs {1}".format(r.stdout, val))
+        self.assertEqual(r.stdout, val, msg)
 
     def script_test(self):
         '''Script upload'''
-        waitsec = 8
+        waitsec = 12
         cik = self.client.cik()
         r = rpc('script', 'files/helloworld.lua', cik)
         self.assertTrue(r.exitcode == 0, 'New script')
         time.sleep(waitsec)
         self._latest(cik, 'helloworld.lua', 'line 1: Hello world!',
-                     'debug output within {} sec'.format(waitsec))
+                     'debug output within {0} sec'.format(waitsec))
         self._latest(cik, 'string_port_alias', 'Hello dataport!',
-                     'dataport write from script within {} sec'.format(waitsec))
-        r = rpc('script', 'files/helloworld2.lua', cik, '--name={}'.format('helloworld.lua'))
+                     'dataport write from script within {0} sec'.format(waitsec))
+        r = rpc('script', 'files/helloworld2.lua', cik, '--name={0}'.format('helloworld.lua'))
         self.assertTrue(r.exitcode == 0, 'Update existing script')
         time.sleep(waitsec)
         self._latest(cik, 'helloworld.lua', 'line 1: Hello world 2!',
-                     'debug output within {} sec'.format(waitsec))
+                     'debug output from updated script within {0} sec'.format(waitsec))
         self._latest(cik, 'string_port_alias', 'Hello dataport 2!',
-                     'dataport write from script within {} sec'.format(waitsec))
+                     'dataport write from updated script within {0} sec'.format(waitsec))
 
     def usage_test(self):
         '''OneP resource usage'''
@@ -522,18 +499,134 @@ Asked for desc: {}\ngot desc: {}'''.format(res.desc, res.info['description']))
         def parse_metric(metric, r):
             self.assertTrue(r.exitcode == 0, 'usage call succeeded')
             self.l(r.stdout)
-            m = re.match(".*{}: (\d+).*".format(metric), r.stdout, re.DOTALL)
-            self.assertTrue(m is not None, 'match metric {} in results'.format(metric))
+            m = re.match(".*{0}: (\d+).*".format(metric), r.stdout, re.DOTALL)
+            self.assertTrue(m is not None, 'match metric {0} in results'.format(metric))
             return int(m.groups()[0])
         r = rpc('usage', self.client.cik(), '--start=10/1/2012T13:04:05')
         dp1 = parse_metric('dataport', r)
         self._create(Resource(self.client.cik(),
                               'dataport',
-                              {'format': 'integer', 'name': 'int_port'},
-                              write=['1'],
-                              record=[[665366400, '2']]))
+                              {'format': 'integer', 'name': 'int_port'}))
         r = rpc('usage', self.client.cik(), '--start=10/1/2012T13:04:05', '--end=now')
         dp2 = parse_metric('dataport', r)
-        self.l("dp1: {} dp2: {}".format(dp1, dp2))
+        self.l("dp1: {0} dp2: {1}".format(dp1, dp2))
         # TODO: why does this not increase consistently?
         self.assertTrue(dp2 > dp1, 'adding dataport added to dataport metric')
+
+    def readmultiple_test(self):
+        '''Read multiple RIDs'''
+        dataports = []
+        strings =  [('2013-07-20T02:40:07', 'a'),
+                    ('2013-07-20T02:50:07', 'b'),
+                    ('2013-07-20T03:00:07', 'c')]
+        integers = [('2013-07-20T02:40:08', 1),
+                    ('2013-07-20T02:50:07', 2),
+                    ('2013-07-20T03:00:08', 3)]
+        floats =   [('2013-07-20T02:40:09', 0.1),
+                    ('2013-07-20T02:50:09', 0.2),
+                    ('2013-07-20T03:00:08', 0.3)]
+        cik = self.client.cik()
+        def rec(fmt, data):
+            r = rpc('record', cik, self.dataports[fmt].rid,
+                *['--value={0},{1}'.format(t, v) for t, v in data])
+            self.assertTrue(r.exitcode == 0)
+
+        rec('string', strings)
+        rec('integer', integers)
+        rec('float', floats)
+
+        rids = [self.dataports[fmt].rid for fmt in ['string', 'integer', 'float']]
+        r = rpc('read', '--start=2013-07-20T3:00:08', '--end=2013-07-20T3:00:08', cik, *rids)
+        self.assertEqual(r.exitcode, 0, 'read with multiple rids')
+        self.assertTrue(r.stdout == '2013-07-20 03:00:08,,3,0.3', 'two readings on one timestamp')
+
+        r = rpc('read', '--start=2013-07-20T2:40:07', '--end=2013-07-20T2:40:09', cik, *rids)
+        self.l(r.stdout)
+        lines = r.stdout.splitlines()
+        self.assertEqual(lines[0], '2013-07-20 02:40:09,,,0.1', 'three timestamps')
+        self.assertEqual(lines[1], '2013-07-20 02:40:08,,1,', 'three timestamps')
+        self.assertEqual(lines[2], '2013-07-20 02:40:07,a,,', 'three timestamps')
+
+        r = rpc('read', '--start=2013-07-20T3:00:09', cik, *rids)
+        self.assertEqual(r.exitcode, 0, 'no data read succeeds')
+        self.assertTrue(r.stdout == '', 'no data')
+
+        rids.reverse()
+        r = rpc('read', '--start=2013-07-20T3:00:08', '--end=2013-07-20T3:00:08', cik, *rids)
+        self.assertTrue(r.stdout == '2013-07-20 03:00:08,0.3,3,', 'rid order reversed')
+
+
+    def _logall(self, r):
+        self.l('stdout: {0}\nstderr: {1}'.format(r.stdout, r.stderr))
+
+    def _stdre(self, r, msg, search, match, stderr=False):
+        std, label = (r.stderr, "stderr") if stderr else (r.stdout, "stdout")
+        if search is not None:
+            self.assertTrue(re.search(search, std, flags=re.MULTILINE) is not None,
+                msg + ' - search\n' + label + ':' + std + '\nsearch re: ' + search)
+        if match is not None:
+            self.assertTrue(re.match(match, std, flags=re.MULTILINE) is not None,
+                msg + ' - match\n' + label + ':' + std + '\nmatch re: ' + match)
+
+    def notok(self, response, msg='', search=None, match=None):
+        if response.exitcode == 0:
+            self._logall(response)
+        self.assertNotEqual(response.exitcode, 0, msg + ' (exit code should not be 0)')
+        self._stdre(response, msg, search=search, match=match, stderr=True)
+
+    def ok(self, response, msg='', search=None, match=None):
+        if response.exitcode != 0:
+            self._logall(response)
+        self.assertEqual(response.exitcode, 0, msg + ' (exit code should be 0)')
+        self._stdre(response, msg, search=search, match=match, stderr=False)
+
+
+    def copy_test(self):
+        '''Test copy and diff commands'''
+
+        cik = self.client.cik()
+
+        r = rpc('diff', cik, self.client.cik())
+        self.ok(r, 'diff with itself, no differences', match='')
+
+        r = rpc('copy', cik, self.portalcik, '--cikonly')
+        self.ok(r, 'copy test client', match=self.RE_RID)
+        copycik = r.stdout
+
+        r = rpc('diff', cik, copycik, '--no-children')
+        self.ok(r, '--no-children, no differences', match='')
+
+        r = rpc('diff', copycik, self.client.cik(), '--no-children')
+        self.ok(r, 'reverse cik, still no differences', match='')
+
+        r = rpc('diff', copycik, cik)
+        self.ok(r, 'diff with children should match', match='')
+
+        newalias = 'newalias'
+        r = rpc('map', cik, self.dataports['string'].rid, newalias)
+        self.ok(r, 'add an alias')
+        r = rpc('diff', copycik, cik)
+        self.ok(r, 'diff notices new alias', search=r'^\+.*' + newalias)
+        r = rpc('diff', cik, copycik)
+        self.ok(r, 'diff notices new alias (reversed)', search=r'^\-.*' + newalias)
+
+        r = rpc('lookup', copycik, 'string_port_alias')
+        self.ok(r, 'lookup copy dataport')
+        copyrid = r.stdout
+        r = rpc('map', copycik, copyrid, newalias)
+        self.ok(r, 'add same alias to copy')
+        r = rpc('diff', cik, copycik)
+        self.ok(r, 'aliases match now', match='')
+        '''
+        r = rpc('copy', cik, cik, '--cikonly')
+        self.ok(r, 'copy client into itself', match=self.RE_RID)
+        innercik = r.stdout
+
+        r = rpc('copy', copycik, copycik, '--cikonly')
+        self.ok(r, 'copy client copy into itself', match=self.RE_RID)
+        innercopycik = r.stdout
+
+        r = rpc('lookup', innercopycik, 'string_port_alias')
+        self.ok(r, 'lookup dataport on inner cik copy', match=self.RE_RID)
+        innercopydataportrid = r.stdout
+        '''
